@@ -1,3 +1,8 @@
+// copyright ############################### //
+// This file is part of the Xtrack Package.  //
+// Copyright (c) CERN, 2021.                 //
+// ######################################### //
+
 #ifndef XTRACK_CAVITY_H
 #define XTRACK_CAVITY_H
 
@@ -11,8 +16,7 @@ void Cavity_track_local_particle(CavityData el, LocalParticle* part0){
         double const   zeta   = LocalParticle_get_zeta(part);
         double const   q      = fabs(LocalParticle_get_q0(part))
                 		    * LocalParticle_get_charge_ratio(part);
-        double         rvv    = LocalParticle_get_rvv(part);
-        double const   tau    = zeta / ( beta0 * rvv );
+        double const   tau    = zeta / beta0;
 
         double const   phase  = DEG2RAD  * CavityData_get_lag(el) -
                                 K_FACTOR * CavityData_get_frequency(el) * tau;
