@@ -14,7 +14,7 @@ import xobjects as xo
 import xpart as xp
 
 from .mad_loader import MadLoader
-from .beam_elements import element_classes, Multipole
+from .beam_elements import element_classes
 from . import beam_elements
 from .beam_elements import Drift
 
@@ -43,11 +43,7 @@ def _is_thick(element):
     return  ((hasattr(element, "isthick") and element.isthick) or
              (isinstance(element, _thick_element_types)))
 
-
-
-
-# missing access to particles._m:
-deg2rad = np.pi / 180.
+DEG2RAD = np.pi / 180.
 
 class AttrDict(dict):
     def __init__(self, *args, **kwargs):
@@ -55,32 +51,32 @@ class AttrDict(dict):
         self.__dict__ = self
 
 
-
 class Line:
 
     @classmethod
     def from_dict(cls, dct, _context=None, _buffer=None, classes=()):
-        class_dict=mk_class_namespace(classes)
+        class_dict = mk_class_namespace(classes)
 
-        _buffer, _ = xo.get_a_buffer(size=8,context=_context, buffer=_buffer)
-        num_elements = len(dct['element_names'])
-        elements = []
-        for ii, el in enumerate(dct["elements"]):
+        _buffer = xo.get_a_buffer(context=_context, buffer=_buffer,size=8)
 
-            if ii % 100 == 0:
-                print(
-                    f'Loading line from dict: {round(ii/num_elements*100):2d}%  ',
-                    end="\r", flush=True)
-
-            eltype = class_dict[el["__class__"]]
-            eldct=el.copy()
-            del eldct['__class__']
-            if hasattr(eltype,'_XoStruct'):
-               newel = eltype.from_dict(eldct,_buffer=_buffer)
-            else:
-               newel = eltype.from_dict(eldct)
-
-            elements.append(newel)
+        if isinstance(dct['elements'], dict):
+            elements = {}
+            num_elements = len(dct['elements'].keys())
+            for ii, (kk, ee) in enumerate(dct['elements'].items()):
+                if ii % 100 == 0:
+                    print('Loading line from dict: '
+                        f'{round(ii/num_elements*100):2d}%  ',end="\r", flush=True)
+                elements[kk] = _deserialize_element(ee, class_dict, _buffer)
+        elif isinstance(dct['elements'], list):
+            elements = []
+            num_elements = len(dct['elements'])
+            for ii, ee in enumerate(dct['elements']):
+                if ii % 100 == 0:
+                    print('Loading line from dict: '
+                        f'{round(ii/num_elements*100):2d}%  ',end="\r", flush=True)
+                elements.append(_deserialize_element(ee, class_dict, _buffer))
+        else:
+            raise ValueError('Field `elements` must be a dict or a list')
 
         self = cls(elements=elements, element_names=dct['element_names'])
 
@@ -89,12 +85,7 @@ class Line:
                                     _context=_buffer.context)
 
         if '_var_manager' in dct.keys():
-            self._init_var_management()
-            manager = self._var_management['manager']
-            for kk in self._var_management['data'].keys():
-                self._var_management['data'][kk].update(
-                                            dct['_var_management_data'][kk])
-            manager.load(dct['_var_manager'])
+            self._init_var_management(dct=dct)
 
         print('Done loading line from dict.           ')
 
@@ -103,7 +94,7 @@ class Line:
     @classmethod
     def from_sixinput(cls, sixinput, classes=()):
         log.warning("\n"
-            "WARNING: xtrack.Line.from_sixinput(sixinput) will be removed in furure versions.\n"
+            "WARNING: xtrack.Line.from_sixinput(sixinput) will be removed in future versions.\n"
             "Please use sixinput.generate_xtrack_line()\n")
         line = sixinput.generate_xtrack_line(classes=classes)
         return line
@@ -144,7 +135,7 @@ class Line:
         line=loader.make_line()
         return line
 
-    def _init_var_management(self):
+    def _init_var_management(self, dct=None):
 
         from collections import defaultdict
         import xdeps as xd
@@ -167,6 +158,14 @@ class Line:
         self._var_management['vref'] = _vref
         self._var_management['fref'] = _fref
 
+        if dct is not None:
+            manager = self._var_management['manager']
+            for kk in self._var_management['data'].keys():
+                self._var_management['data'][kk].update(
+                                            dct['_var_management_data'][kk])
+            manager.load(dct['_var_manager'])
+
+
     @property
     def vars(self):
         if self._var_management is not None:
@@ -317,15 +316,38 @@ class Line:
     def __len__(self):
         return len(self.element_names)
 
+    def copy(self, _context=None, _buffer=None):
+
+        elements = {nn: ee.copy(_context=_context, _buffer=_buffer)
+                                    for nn, ee in self.element_dict.items()}
+        element_names = [nn for nn in self.element_names]
+
+        out = self.__class__(elements=elements, element_names=element_names)
+
+        if self.particle_ref is not None:
+            out.particle_ref = self.particle_ref.copy(
+                                        _context=_context, _buffer=_buffer)
+
+        if self._var_management is not None:
+            out._init_var_management(dct=self._var_management_to_dict())
+
+        return out
+
+
+    def _var_management_to_dict(self):
+        out = {}
+        out['_var_management_data'] = deepcopy(self._var_management['data'])
+        out['_var_manager'] = self._var_management['manager'].dump()
+        return out
+
     def to_dict(self):
         out = {}
-        out["elements"] = [el.to_dict() for el in self.elements]
+        out["elements"] = {k: el.to_dict() for k, el in self.element_dict.items()}
         out["element_names"] = self.element_names[:]
         if self.particle_ref is not None:
             out['particle_ref'] = self.particle_ref.to_dict()
         if self._var_management is not None:
-            out['_var_management_data'] = deepcopy(self._var_management['data'])
-            out['_var_manager'] = self._var_management['manager'].dump()
+            out.update(self._var_management_to_dict())
         return out
 
     def to_pandas(self):
@@ -345,9 +367,6 @@ class Line:
         })
         return elements_df
 
-    def copy(self):
-        return self.__class__.from_dict(self.to_dict())
-
     def insert_element(self, index=None, element=None, name=None, at_s=None,
                        s_tol=1e-6):
 
@@ -500,10 +519,13 @@ class Line:
         else:
             return s
 
-    def remove_inactive_multipoles(self, inplace=False):
+    def remove_inactive_multipoles(self, inplace=True):
 
         self._frozen_check()
 
+        if not inplace:
+            raise NotImplementedError
+
         newline = Line(elements=[], element_names=[])
 
         for ee, nn in zip(self.elements, self.element_names):
@@ -513,16 +535,17 @@ class Line:
                     continue
             newline.append_element(ee, nn)
 
-        if inplace:
-            self.element_names = newline.element_names
-            return self
-        else:
-            return newline
 
-    def remove_zero_length_drifts(self, inplace=False):
+        self.element_names = newline.element_names
+        return self
+
+    def remove_zero_length_drifts(self, inplace=True):
 
         self._frozen_check()
 
+        if not inplace:
+            raise NotImplementedError
+
         newline = Line(elements=[], element_names=[])
 
         for ee, nn in zip(self.elements, self.element_names):
@@ -531,20 +554,20 @@ class Line:
                     continue
             newline.append_element(ee, nn)
 
-        if inplace:
-            self.element_names = newline.element_names
-            return self
-        else:
-            return newline
+        self.element_names = newline.element_names
+        return self
 
-    def merge_consecutive_drifts(self, inplace=False):
+    def merge_consecutive_drifts(self, inplace=True):
 
         self._frozen_check()
 
+        if not inplace:
+            raise NotImplementedError
+
         newline = Line(elements=[], element_names=[])
 
-        for ee, nn in zip(self.elements, self.element_names):
-            if len(newline.elements) == 0:
+        for ii, (ee, nn) in enumerate(zip(self.elements, self.element_names)):
+            if ii == 0:
                 newline.append_element(ee, nn)
                 continue
 
@@ -553,20 +576,16 @@ class Line:
                 prev_ee = newline.element_dict[prev_nn]
                 if _is_drift(prev_ee):
                     prev_ee.length += ee.length
-                    newline.element_names[-1] = prev_nn
                 else:
                     newline.append_element(ee, nn)
             else:
                 newline.append_element(ee, nn)
 
-        if inplace:
-            self.element_dict.update(newline.element_dict)
-            self.element_names = newline.element_names
-            return self
-        else:
-            return newline
+        self.element_dict.update(newline.element_dict)
+        self.element_names = newline.element_names
+        return self
 
-    def merge_consecutive_multipoles(self, inplace=False):
+    def merge_consecutive_multipoles(self, inplace=True):
 
         self._frozen_check()
         if self._var_management is not None:
@@ -574,6 +593,9 @@ class Line:
                                       ' available when deferred expressions are'
                                       ' used')
 
+        if not inplace:
+            raise NotImplementedError
+
         newline = Line(elements=[], element_names=[])
 
         for ee, nn in zip(self.elements, self.element_names):
@@ -612,12 +634,9 @@ class Line:
             else:
                 newline.append_element(ee, nn)
 
-        if inplace:
-            self.element_dict.update(newline.element_dict)
-            self.element_names = newline.element_names
-            return self
-        else:
-            return newline
+        self.element_dict.update(newline.element_dict)
+        self.element_names = newline.element_names
+        return self
 
     def get_elements_of_type(self, types):
         if not hasattr(types, "__iter__"):
@@ -738,3 +757,11 @@ mathfunctions.floor=math.floor
 mathfunctions.ceil=math.ceil
 mathfunctions.round=np.round
 mathfunctions.frac=lambda x: (x%1)
+
+def _deserialize_element(el, class_dict, _buffer):
+    eldct = el.copy()
+    eltype = class_dict[eldct.pop('__class__')]
+    if hasattr(eltype, '_XoStruct'):
+        return eltype.from_dict(eldct, _buffer=_buffer)
+    else:
+        return eltype.from_dict(eldct)
